@@ -33,7 +33,11 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_bri.c 7485 2009-11-04 21:45:55Z tzafrir $";
+static const char rcsid[] = "$Id: card_bri.c 10390 2012-01-02 14:07:14Z tzafrir $";
+
+#ifndef	DAHDI_SIG_HARDHDLC
+#error Cannot build BRI without HARDHDLC supprt
+#endif
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -56,6 +60,7 @@ enum xhfc_states {
 	ST_NT_DEACTIVTING	= 4,	/* G4	*/
 };
 
+#ifdef CONFIG_PROC_FS
 static const char *xhfc_state_name(bool is_nt, enum xhfc_states state)
 {
 	const char	*p;
@@ -80,18 +85,19 @@ static const char *xhfc_state_name(bool is_nt, enum xhfc_states state)
 	};
 #undef	_E
 	if(is_nt) {
-		if ((state < ST_RESET) || (state > ST_NT_DEACTIVTING))
+		if (state > ST_NT_DEACTIVTING)
 			p = "NT ???";
 		else
 			p = nt_names[state];
 	} else {
-		if ((state < ST_RESET) || (state > ST_TE_LOST_FRAMING))
+		if (state > ST_TE_LOST_FRAMING)
 			p = "TE ???";
 		else
 			p = te_names[state];
 	}
 	return p;
 }
+#endif
 
 /* xhfc Layer1 physical commands */
 #define HFC_L1_ACTIVATE_TE		0x01
@@ -127,25 +133,10 @@ typedef union {
 #define	REG30_LOST	3	/* in polls */
 #define	DCHAN_LOST	15000	/* in ticks */
 
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-#define	BRI_DCHAN_SIGCAP	(			  \
-					DAHDI_SIG_EM	| \
-					DAHDI_SIG_CLEAR	| \
-					DAHDI_SIG_FXSLS	| \
-					DAHDI_SIG_FXSGS	| \
-					DAHDI_SIG_FXSKS	| \
-					DAHDI_SIG_FXOLS	| \
-					DAHDI_SIG_FXOGS	| \
-					DAHDI_SIG_FXOKS	| \
-					DAHDI_SIG_CAS	| \
-					DAHDI_SIG_SF	  \
-				)
-#else
 #define	BRI_DCHAN_SIGCAP	DAHDI_SIG_HARDHDLC
-#endif
 #define	BRI_BCHAN_SIGCAP	(DAHDI_SIG_CLEAR | DAHDI_SIG_DACS)
 
-#define	IS_NT(xpd)		((xpd)->direction == TO_PHONE)
+#define	IS_NT(xpd)		(PHONEDEV(xpd).direction == TO_PHONE)
 #define	BRI_PORT(xpd)		((xpd)->addr.subunit)
 
 /* shift in PCM highway */
@@ -157,10 +148,14 @@ typedef union {
 static int write_state_register(xpd_t *xpd, byte value);
 static bool bri_packet_is_valid(xpacket_t *pack);
 static void bri_packet_dump(const char *msg, xpacket_t *pack);
+#ifdef	CONFIG_PROC_FS
 static int proc_bri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data);
-static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc);
-static int bri_chanconfig(struct dahdi_chan *chan, int sigtype);
-static int bri_startup(struct dahdi_span *span);
+#endif
+static int bri_spanconfig(struct file *file, struct dahdi_span *span,
+			  struct dahdi_lineconfig *lc);
+static int bri_chanconfig(struct file *file, struct dahdi_chan *chan,
+			  int sigtype);
+static int bri_startup(struct file *file, struct dahdi_span *span);
 static int bri_shutdown(struct dahdi_span *span);
 
 #define	PROC_REGISTER_FNAME	"slics"
@@ -212,13 +207,7 @@ struct BRI_priv_data {
 	/*
 	 * D-Chan: buffers + extra state info.
 	 */
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	int				dchan_r_idx;
-	byte				dchan_rbuf[DCHAN_BUFSIZE];
-#else
 	atomic_t			hdlc_pending;
-#endif
-	byte				dchan_tbuf[DCHAN_BUFSIZE];
 	bool				txframe_begin;
 
 	uint				tick_counter;
@@ -340,14 +329,12 @@ static void layer1_state(xpd_t *xpd, bool up)
 static void te_activation(xpd_t *xpd, bool on)
 {
 	struct BRI_priv_data	*priv;
-	xbus_t			*xbus;
 	byte			curr_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	curr_state = priv->state_register.bits.v_su_sta;
-	xbus = xpd->xbus;
 	XPD_DBG(SIGNAL, xpd, "%s\n", (on)?"ON":"OFF");
 	if(on) {
 		if(curr_state == ST_TE_DEACTIVATED) {
@@ -385,14 +372,12 @@ static void te_activation(xpd_t *xpd, bool on)
 static void nt_activation(xpd_t *xpd, bool on)
 {
 	struct BRI_priv_data	*priv;
-	xbus_t			*xbus;
 	byte			curr_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	curr_state = priv->state_register.bits.v_su_sta;
-	xbus = xpd->xbus;
 	XPD_DBG(SIGNAL, xpd, "%s\n", (on)?"ON":"OFF");
 	if(on) {
 		switch(curr_state) {
@@ -436,22 +421,6 @@ static void nt_activation(xpd_t *xpd, bool on)
 /*
  * D-Chan receive
  */
-static void bri_hdlc_abort(xpd_t *xpd, struct dahdi_chan *dchan, int event)
-{
-	struct BRI_priv_data	*priv;
-
-	priv = xpd->priv;
-	BUG_ON(!priv);
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	if(debug & DBG_COMMANDS)
-		dump_hex_buf(xpd, "D-Chan(abort) RX: dchan_rbuf",
-			priv->dchan_rbuf, priv->dchan_r_idx);
-	priv->dchan_r_idx = 0;
-#else
-	dahdi_hdlc_abort(dchan, event);
-#endif
-}
-
 static int bri_check_stat(xpd_t *xpd, struct dahdi_chan *dchan, byte *buf, int len)
 {
 	struct BRI_priv_data	*priv;
@@ -459,21 +428,11 @@ static int bri_check_stat(xpd_t *xpd, struct dahdi_chan *dchan, byte *buf, int l
 
 	priv = xpd->priv;
 	BUG_ON(!priv);
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	if(priv->dchan_r_idx < 4) {
-		XPD_NOTICE(xpd, "D-Chan RX short frame (dchan_r_idx=%d)\n",
-			priv->dchan_r_idx);
-		dump_hex_buf(xpd, "D-Chan RX:    current packet", buf, len);
-		bri_hdlc_abort(xpd, dchan, DAHDI_EVENT_ABORT);
-		return -EPROTO;
-	}
-#else
 	if(len <= 0) {
 		XPD_NOTICE(xpd, "D-Chan RX DROP: short frame (len=%d)\n", len);
-		bri_hdlc_abort(xpd, dchan, DAHDI_EVENT_ABORT);
+		dahdi_hdlc_abort(dchan, DAHDI_EVENT_ABORT);
 		return -EPROTO;
 	}
-#endif
 	status = buf[len-1];
 	if(status) {
 		int	event = DAHDI_EVENT_ABORT;
@@ -485,138 +444,14 @@ static int bri_check_stat(xpd_t *xpd, struct dahdi_chan *dchan, byte *buf, int l
 			event = DAHDI_EVENT_BADFCS;
 		}
 		dump_hex_buf(xpd, "D-Chan RX:    current packet", buf, len);
-		bri_hdlc_abort(xpd, dchan, event);
+		dahdi_hdlc_abort(dchan, event);
 		return -EPROTO;
 	}
 	return 0;
 }
 
-int bri_hdlc_putbuf(xpd_t *xpd, struct dahdi_chan *dchan, unsigned char *buf, int len)
-{
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	struct BRI_priv_data	*priv;
-	byte			*dchan_buf;
-	byte			*dst;
-	int			idx;
-
-	priv = xpd->priv;
-	BUG_ON(!priv);
-	dchan_buf = dchan->readchunk;
-	idx = priv->dchan_r_idx;
-	if(idx + len >= DCHAN_BUFSIZE) {
-		XPD_ERR(xpd, "D-Chan RX overflow: %d\n", idx);
-		dump_hex_buf(xpd, "    current packet", buf, len);
-		dump_hex_buf(xpd, "    dchan_buf", dchan_buf, idx);
-		return -ENOSPC;
-	}
-	dst = dchan_buf + idx;
-	idx += len;
-	priv->dchan_r_idx = idx;
-	memcpy(dst, buf, len);
-#else
-	dahdi_hdlc_putbuf(dchan, buf, len);
-#endif
-	return 0;
-}
-
-void bri_hdlc_finish(xpd_t *xpd, struct dahdi_chan *dchan)
-{
-	struct BRI_priv_data	*priv;
-
-	priv = xpd->priv;
-	BUG_ON(!priv);
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	dchan->bytes2receive = priv->dchan_r_idx - 1;
-	dchan->eofrx = 1;
-#else
-	dahdi_hdlc_finish(dchan);
-#endif
-}
-
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
 static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 {
-	xbus_t			*xbus;
-	struct BRI_priv_data	*priv;
-	byte			*src;
-	byte			*dst;
-	byte			*dchan_buf;
-	struct dahdi_chan		*dchan;
-	uint			len;
-	bool			eoframe;
-	int			idx;
-	int			ret = 0;
-
-	src = REG_XDATA(regcmd);
-	len = regcmd->bytes;
-	eoframe = regcmd->eoframe;
-	if(len <= 0)
-		return 0;
-	if(!SPAN_REGISTERED(xpd)) /* Nowhere to copy data */
-		return 0;
-	BUG_ON(!xpd);
-	priv = xpd->priv;
-	BUG_ON(!priv);
-	xbus = xpd->xbus;
-#ifdef XPP_DEBUGFS
-	xbus_log(xbus, xpd, 0, regcmd, sizeof(reg_cmd_t));		/* 0 = RX */
-#endif
-	dchan = XPD_CHAN(xpd, 2);
-	if(!IS_OFFHOOK(xpd, 2)) {	/* D-chan is used? */
-		static int rate_limit;
-
-		if((rate_limit++ % 1000) == 0)
-			XPD_DBG(SIGNAL, xpd, "D-Chan unused\n");
-		dchan->bytes2receive = 0;
-		dchan->bytes2transmit = 0;
-		goto out;
-	}
-	dchan_buf = dchan->readchunk;
-	idx = priv->dchan_r_idx;
-	if(idx + len >= DCHAN_BUFSIZE) {
-		XPD_ERR(xpd, "D-Chan RX overflow: %d\n", idx);
-		dump_hex_buf(xpd, "    current packet", src, len);
-		dump_hex_buf(xpd, "    dchan_buf", dchan_buf, idx);
-		ret = -ENOSPC;
-		if(eoframe)
-			goto drop;
-		goto out;
-	}
-	dst = dchan_buf + idx;
-	idx += len;
-	priv->dchan_r_idx = idx;
-	memcpy(dst, src, len);
-	if(!eoframe)
-		goto out;
-	if(idx < 4) {
-		XPD_NOTICE(xpd, "D-Chan RX short frame (idx=%d)\n", idx);
-		dump_hex_buf(xpd, "D-Chan RX:    current packet", src, len);
-		dump_hex_buf(xpd, "D-Chan RX:    chan_buf", dchan_buf, idx);
-		ret = -EPROTO;
-		goto drop;
-	}
-	if((ret = bri_check_stat(xpd, dchan, dchan_buf, idx)) < 0)
-		goto drop;
-	if(debug)
-		dump_dchan_packet(xpd, 0, dchan_buf, idx /* - 3 */);	/* Print checksum? */
-	/* 
-	 * Tell Dahdi that we received idx-1 bytes. They include the data and a 2-byte checksum.
-	 * The last byte (that we don't pass on) is 0 if the checksum is correct. If it were wrong,
-	 * we would drop the packet in the "if(dchan_buf[idx-1])" above.
-	 */
-	dchan->bytes2receive = idx - 1;
-	dchan->eofrx = 1;
-	priv->dchan_rx_counter++;
-	priv->dchan_norx_ticks = 0;
-drop:
-	priv->dchan_r_idx = 0;
-out:
-	return ret;
-}
-#else
-static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
-{
-	xbus_t			*xbus;
 	struct BRI_priv_data	*priv;
 	byte			*src;
 	struct dahdi_chan	*dchan;
@@ -634,26 +469,16 @@ static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	xbus = xpd->xbus;
-#ifdef XPP_DEBUGFS
-	xbus_log(xbus, xpd, 0, regcmd, sizeof(reg_cmd_t));		/* 0 = RX */
-#endif
 	dchan = XPD_CHAN(xpd, 2);
 	if(!IS_OFFHOOK(xpd, 2)) {	/* D-chan is used? */
 		static int rate_limit;
 
 		if((rate_limit++ % 1000) == 0)
 			XPD_DBG(SIGNAL, xpd, "D-Chan unused\n");
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-		dchan->bytes2receive = 0;
-		dchan->bytes2transmit = 0;
-#endif
 		goto out;
 	}
 	XPD_DBG(GENERAL, xpd, "D-Chan RX: eoframe=%d len=%d\n", eoframe, len);
-	ret = bri_hdlc_putbuf(xpd, dchan, src, (eoframe) ? len - 1 : len);
-	if(ret < 0)
-		goto out;
+	dahdi_hdlc_putbuf(dchan, src, (eoframe) ? len - 1 : len);
 	if(!eoframe)
 		goto out;
 	if((ret = bri_check_stat(xpd, dchan, src, len)) < 0)
@@ -663,18 +488,16 @@ static int rx_dchan(xpd_t *xpd, reg_cmd_t *regcmd)
 	 * The last byte (that we don't pass on) is 0 if the checksum is correct. If it were wrong,
 	 * we would drop the packet in the "if(src[len-1])" above.
 	 */
-	bri_hdlc_finish(xpd, dchan);
+	dahdi_hdlc_finish(dchan);
 	priv->dchan_rx_counter++;
 	priv->dchan_norx_ticks = 0;
 out:
 	return ret;
 }
-#endif
 
 /*
  * D-Chan transmit
  */
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
 /* DAHDI calls this when it has data it wants to send to the HDLC controller */
 static void bri_hdlc_hard_xmit(struct dahdi_chan *chan)
 {
@@ -689,85 +512,129 @@ static void bri_hdlc_hard_xmit(struct dahdi_chan *chan)
 		atomic_inc(&priv->hdlc_pending);
 	}
 }
-#endif
 
-int bri_hdlc_getbuf(struct dahdi_chan *dchan, unsigned char *buf, unsigned int *size)
+static int send_dchan_frame(xpd_t *xpd, xframe_t *xframe, bool is_eof)
 {
-	int			len = *size;
-	int			eoframe;
-
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	len = dchan->bytes2transmit;	/* dchan's hdlc package len */
-	if(len > *size)
-		len = *size;		/* Silent truncation */
-	eoframe = dchan->eoftx;		/* dchan's end of frame */
-	dchan->bytes2transmit = 0;
-	dchan->eoftx = 0;
-	dchan->bytes2receive = 0;
-	dchan->eofrx = 0;
-#else
-	eoframe = dahdi_hdlc_getbuf(dchan, buf, &len);
-#endif
-	*size = len;
-	return eoframe;
-}
-
-static int tx_dchan(xpd_t *xpd)
-{
-	struct BRI_priv_data	*priv;
-	struct dahdi_chan	*dchan;
-	int			len;
-	int			eoframe;
-	int			ret;
+	struct BRI_priv_data *priv;
+	int ret;
 
+	XPD_DBG(COMMANDS, xpd, "eoframe=%d\n", is_eof);
 	priv = xpd->priv;
-	BUG_ON(!priv);
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
-	if(atomic_read(&priv->hdlc_pending) == 0)
-		return 0;
-#endif
-	if(!SPAN_REGISTERED(xpd) || !(xpd->span.flags & DAHDI_FLAG_RUNNING))
-		return 0;
-	dchan = XPD_CHAN(xpd, 2);
-	len = ARRAY_SIZE(priv->dchan_tbuf);
-	if(len > MULTIBYTE_MAX_LEN)
-		len = MULTIBYTE_MAX_LEN;
-	eoframe = bri_hdlc_getbuf(dchan, priv->dchan_tbuf, &len);
-	if(len <= 0)
-		return 0; /* Nothing to transmit on D channel */
-	if(len > MULTIBYTE_MAX_LEN) {
-		XPD_ERR(xpd, "%s: len=%d. need to split. Unimplemented.\n", __FUNCTION__, len);
-		dump_hex_buf(xpd, "D-Chan TX:", priv->dchan_tbuf, len);
-		return -EINVAL;
-	}
-	if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags) && !test_bit(HFC_L1_ACTIVATING, &priv->l1_flags)) {
+	if (!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags)
+			&& !test_bit(HFC_L1_ACTIVATING, &priv->l1_flags)) {
 		XPD_DBG(SIGNAL, xpd, "Want to transmit: Kick D-Channel transmiter\n");
-		if(xpd->direction == TO_PSTN)
+		if (!IS_NT(xpd))
 			te_activation(xpd, 1);
 		else
 			nt_activation(xpd, 1);
 	}
-	if(debug)
-		dump_dchan_packet(xpd, 1, priv->dchan_tbuf, len);
-	if(eoframe)
-		priv->txframe_begin = 1;
-	else
-		priv->txframe_begin = 0;
-	XPD_DBG(COMMANDS, xpd, "eoframe=%d len=%d\n", eoframe, len);
-	ret = send_multibyte_request(xpd->xbus, xpd->addr.unit, xpd->addr.subunit,
-			eoframe, priv->dchan_tbuf, len);
-	if(ret < 0)
-		XPD_NOTICE(xpd, "%s: failed sending xframe\n", __FUNCTION__);
-	if(eoframe) {
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
+	dump_xframe("send_dchan_frame", xpd->xbus, xframe, debug);
+	ret = send_cmd_frame(xpd->xbus, xframe);
+	if (ret < 0)
+		XPD_ERR(xpd, "%s: failed sending xframe\n", __func__);
+	if (is_eof) {
 		atomic_dec(&priv->hdlc_pending);
-#endif
 		priv->dchan_tx_counter++;
-	}
+		priv->txframe_begin = 1;
+	} else
+		priv->txframe_begin = 0;
 	priv->dchan_notx_ticks = 0;
 	return ret;
 }
 
+/*
+ * Fill a single multibyte REGISTER_REQUEST
+ */
+static void fill_multibyte(xpd_t *xpd, xpacket_t *pack, bool eoframe,
+		char *buf, int len)
+{
+	reg_cmd_t *reg_cmd;
+	char *p;
+
+	XPACKET_INIT(pack, GLOBAL, REGISTER_REQUEST, xpd->xbus_idx, 0, 0);
+	XPACKET_LEN(pack) = RPACKET_SIZE(GLOBAL, REGISTER_REQUEST);
+	reg_cmd = &RPACKET_FIELD(pack, GLOBAL, REGISTER_REQUEST, reg_cmd);
+	reg_cmd->bytes = len;
+	reg_cmd->is_multibyte = 1;
+	reg_cmd->portnum = xpd->addr.subunit;
+	reg_cmd->eoframe = eoframe;
+	p = REG_XDATA(reg_cmd);
+	memcpy(p, buf, len);
+	if (debug)
+		dump_dchan_packet(xpd, 1, p, len);
+}
+
+/*
+ * Transmit available D-Channel frames
+ *
+ * - FPGA firmware expect to get this as a sequence of REGISTER_REQUEST
+ *   multibyte commands.
+ * - The payload of each command is limited to MULTIBYTE_MAX_LEN bytes.
+ * - We batch several REGISTER_REQUEST packets into a single xframe.
+ * - The xframe is terminated when we get a bri "end of frame"
+ *   or when the xframe is full (should not happen).
+ */
+static int tx_dchan(xpd_t *xpd)
+{
+	struct BRI_priv_data *priv;
+	xframe_t *xframe;
+	xpacket_t *pack;
+	int packet_count;
+	int eoframe;
+	int ret;
+
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	if (atomic_read(&priv->hdlc_pending) == 0)
+		return 0;
+	if (!SPAN_REGISTERED(xpd) ||
+			!(PHONEDEV(xpd).span.flags & DAHDI_FLAG_RUNNING))
+		return 0;
+	/* Allocate frame */
+	xframe = ALLOC_SEND_XFRAME(xpd->xbus);
+	if (!xframe) {
+		XPD_NOTICE(xpd, "%s: failed to allocate new xframe\n",
+			__func__);
+		return -ENOMEM;
+	}
+	for (packet_count = 0, eoframe = 0; !eoframe; packet_count++) {
+		int packet_len = RPACKET_SIZE(GLOBAL, REGISTER_REQUEST);
+		char buf[MULTIBYTE_MAX_LEN];
+		int len = MULTIBYTE_MAX_LEN;
+
+		/* Reserve packet */
+		pack = xframe_next_packet(xframe, packet_len);
+		if (!pack) {
+			BUG_ON(!packet_count);
+			/*
+			 * A split. Send what we currently have.
+			 */
+			XPD_NOTICE(xpd,
+				"%s: xframe is full (%d packets)\n",
+				__func__, packet_count);
+			break;
+		}
+		/* Get data from DAHDI */
+		eoframe = dahdi_hdlc_getbuf(XPD_CHAN(xpd, 2), buf, &len);
+		if (len <= 0) {
+			/*
+			 * Already checked priv->hdlc_pending,
+			 * should never get here.
+			 */
+			if (printk_ratelimit())
+				XPD_ERR(xpd,
+					"%s: hdlc_pending, but nothing to transmit?\n",
+					__func__);
+			FREE_SEND_XFRAME(xpd->xbus, xframe);
+			return -EINVAL;
+		}
+		BUG_ON(len > MULTIBYTE_MAX_LEN);
+		fill_multibyte(xpd, pack, eoframe != 0, buf, len);
+	}
+	return send_dchan_frame(xpd, xframe, eoframe != 0);
+	return ret;
+}
+
 /*---------------- BRI: Methods -------------------------------------------*/
 
 static void bri_proc_remove(xbus_t *xbus, xpd_t *xpd)
@@ -797,14 +664,12 @@ static int bri_proc_create(xbus_t *xbus, xpd_t *xpd)
 	priv->bri_info = create_proc_read_entry(PROC_BRI_INFO_FNAME, 0444, xpd->proc_xpd_dir, proc_bri_info_read, xpd);
 	if(!priv->bri_info) {
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", PROC_BRI_INFO_FNAME);
-		goto err;
+		bri_proc_remove(xbus, xpd);
+		return -EINVAL;
 	}
 	SET_PROC_DIRENTRY_OWNER(priv->bri_info);
 #endif
 	return 0;
-err:
-	bri_proc_remove(xbus, xpd);
-	return -EINVAL;
 }
 
 static xpd_t *BRI_card_new(xbus_t *xbus, int unit, int subunit, const xproto_table_t *proto_table,
@@ -821,7 +686,7 @@ static xpd_t *BRI_card_new(xbus_t *xbus, int unit, int subunit, const xproto_tab
 	xpd = xpd_alloc(xbus, unit, subunit, subtype, subunits, sizeof(struct BRI_priv_data), proto_table, channels);
 	if(!xpd)
 		return NULL;
-	xpd->direction = (to_phone) ? TO_PHONE : TO_PSTN;
+	PHONEDEV(xpd).direction = (to_phone) ? TO_PHONE : TO_PSTN;
 	xpd->type_name = (to_phone) ? "BRI_NT" : "BRI_TE";
 	if(bri_proc_create(xbus, xpd) < 0)
 		goto err;
@@ -848,15 +713,35 @@ static int BRI_card_init(xbus_t *xbus, xpd_t *xpd)
 
 static int BRI_card_remove(xbus_t *xbus, xpd_t *xpd)
 {
-	struct BRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
-	priv = xpd->priv;
 	XPD_DBG(GENERAL, xpd, "\n");
 	bri_proc_remove(xbus, xpd);
 	return 0;
 }
 
+static const struct dahdi_span_ops BRI_span_ops = {
+	.owner = THIS_MODULE,
+	.spanconfig = bri_spanconfig,
+	.chanconfig = bri_chanconfig,
+	.startup = bri_startup,
+	.shutdown = bri_shutdown,
+	.hdlc_hard_xmit = bri_hdlc_hard_xmit,
+	.open = xpp_open,
+	.close = xpp_close,
+	.hooksig = xpp_hooksig,	/* Only with RBS bits */
+	.ioctl = xpp_ioctl,
+	.maint = xpp_maint,
+	.echocan_create = xpp_echocan_create,
+	.echocan_name = xpp_echocan_name,
+	.assigned = xpp_span_assigned,
+#ifdef	DAHDI_SYNC_TICK
+	.sync_tick = dahdi_sync_tick,
+#endif
+#ifdef	CONFIG_DAHDI_WATCHDOG
+	.watchdog = xpp_watchdog,
+#endif
+};
+
 static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
@@ -872,10 +757,10 @@ static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 		/* Nothing to do yet */
 		return 0;
 	}
-	xpd->span.spantype = "BRI";
-	xpd->span.linecompat = DAHDI_CONFIG_AMI | DAHDI_CONFIG_CCS;
-	xpd->span.deflaw = DAHDI_LAW_ALAW;
-	BIT_SET(xpd->digital_signalling, 2);	/* D-Channel */
+	PHONEDEV(xpd).span.spantype = "BRI";
+	PHONEDEV(xpd).span.linecompat = DAHDI_CONFIG_AMI | DAHDI_CONFIG_CCS;
+	PHONEDEV(xpd).span.deflaw = DAHDI_LAW_ALAW;
+	BIT_SET(PHONEDEV(xpd).digital_signalling, 2);	/* D-Channel */
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
 
@@ -889,48 +774,28 @@ static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 			cur_chan->sigcap = BRI_DCHAN_SIGCAP;
 			clear_bit(DAHDI_FLAGBIT_HDLC, &cur_chan->flags);
 			priv->txframe_begin = 1;
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-			priv->dchan_r_idx = 0;
-			set_bit(DAHDI_FLAGBIT_BRIDCHAN, &cur_chan->flags);
-			/* Setup big buffers for D-Channel rx/tx */
-			cur_chan->readchunk = priv->dchan_rbuf;
-			cur_chan->writechunk = priv->dchan_tbuf;
-
-			cur_chan->maxbytes2transmit = MULTIBYTE_MAX_LEN;
-			cur_chan->bytes2transmit = 0;
-			cur_chan->bytes2receive = 0;
-#else
 			atomic_set(&priv->hdlc_pending, 0);
-#endif
 		} else {
 			cur_chan->sigcap = BRI_BCHAN_SIGCAP;
 		}
 	}
-	CALL_XMETHOD(card_pcm_recompute, xbus, xpd, 0);
-	xpd->span.spanconfig = bri_spanconfig;
-	xpd->span.chanconfig = bri_chanconfig;
-	xpd->span.startup = bri_startup;
-	xpd->span.shutdown = bri_shutdown;
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
-	xpd->span.hdlc_hard_xmit = bri_hdlc_hard_xmit;
-#endif
+	CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
+	PHONEDEV(xpd).span.ops = &BRI_span_ops;
 	return 0;
 }
 
 static int BRI_card_dahdi_postregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
-	struct BRI_priv_data	*priv;
 	
 	BUG_ON(!xpd);
 	xbus = xpd->xbus;
-	priv = xpd->priv;
 	BUG_ON(!xbus);
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"on":"off");
 	return(0);
 }
 
-static int BRI_card_hooksig(xbus_t *xbus, xpd_t *xpd, int pos, enum dahdi_txsig txsig)
+static int BRI_card_hooksig(xpd_t *xpd, int pos, enum dahdi_txsig txsig)
 {
 	LINE_DBG(SIGNAL, xpd, pos, "%s\n", txsig2str(txsig));
 	return 0;
@@ -1125,16 +990,13 @@ static int BRI_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long a
 
 static int BRI_card_open(xpd_t *xpd, lineno_t pos)
 {
-	struct BRI_priv_data	*priv;
-
 	BUG_ON(!xpd);
-	priv = xpd->priv;
 	if(pos == 2) {
 		LINE_DBG(SIGNAL, xpd, pos, "OFFHOOK the whole span\n");
-		BIT_SET(xpd->offhook_state, 0);
-		BIT_SET(xpd->offhook_state, 1);
-		BIT_SET(xpd->offhook_state, 2);
-		CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 0);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 1);
+		BIT_SET(PHONEDEV(xpd).offhook_state, 2);
+		CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	}
 	return 0;
 }
@@ -1142,21 +1004,12 @@ static int BRI_card_open(xpd_t *xpd, lineno_t pos)
 static int BRI_card_close(xpd_t *xpd, lineno_t pos)
 {
 	/* Clear D-Channel pending data */
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-	struct dahdi_chan	*chan = XPD_CHAN(xpd, pos);
-
-	/* Clear D-Channel pending data */
-	chan->bytes2receive = 0;
-	chan->eofrx = 0;
-	chan->bytes2transmit = 0;
-	chan->eoftx = 0;
-#endif
 	if(pos == 2) {
 		LINE_DBG(SIGNAL, xpd, pos, "ONHOOK the whole span\n");
-		BIT_CLR(xpd->offhook_state, 0);
-		BIT_CLR(xpd->offhook_state, 1);
-		BIT_CLR(xpd->offhook_state, 2);
-		CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 0);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 1);
+		BIT_CLR(PHONEDEV(xpd).offhook_state, 2);
+		CALL_PHONE_METHOD(card_pcm_recompute, xpd, 0);
 	}
 	return 0;
 }
@@ -1164,9 +1017,11 @@ static int BRI_card_close(xpd_t *xpd, lineno_t pos)
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
-static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
+static int bri_spanconfig(struct file *file, struct dahdi_span *span,
+			  struct dahdi_lineconfig *lc)
 {
-	xpd_t		*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	const char	*framingstr = "";
 	const char	*codingstr = "";
 	const char	*crcstr = "";
@@ -1196,6 +1051,8 @@ static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
 		framingstr, codingstr, crcstr,
 		lc->lineconfig,
 		lc->sync);
+	PHONEDEV(xpd).timing_priority = lc->sync;
+	elect_syncer("BRI-spanconfig");
 	/*
 	 * FIXME: validate
 	 */
@@ -1208,7 +1065,8 @@ static int bri_spanconfig(struct dahdi_span *span, struct dahdi_lineconfig *lc)
  * Called from dahdi with spinlock held on chan. Must not call back
  * dahdi functions.
  */
-static int bri_chanconfig(struct dahdi_chan *chan, int sigtype)
+static int bri_chanconfig(struct file *file, struct dahdi_chan *chan,
+			  int sigtype)
 {
 	DBG(GENERAL, "channel %d (%s) -> %s\n", chan->channo, chan->name, sig2str(sigtype));
 	// FIXME: sanity checks:
@@ -1220,9 +1078,10 @@ static int bri_chanconfig(struct dahdi_chan *chan, int sigtype)
 /*
  * Called only for 'span' keyword in /etc/dahdi/system.conf
  */
-static int bri_startup(struct dahdi_span *span)
+static int bri_startup(struct file *file, struct dahdi_span *span)
 {
-	xpd_t			*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct BRI_priv_data	*priv;
 	struct dahdi_chan	*dchan;
 
@@ -1235,7 +1094,7 @@ static int bri_startup(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "STARTUP\n");
 	// Turn on all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 1);
+	CALL_PHONE_METHOD(card_state, xpd, 1);
 	if(SPAN_REGISTERED(xpd)) {
 		dchan = XPD_CHAN(xpd, 2);
 		span->flags |= DAHDI_FLAG_RUNNING;
@@ -1245,9 +1104,6 @@ static int bri_startup(struct dahdi_span *span)
 		 *
 		 * Don't Get Mad, Get Even:  Now we override dahdi :-)
 		 */
-#ifdef	CONFIG_DAHDI_BRI_DCHANS
-		set_bit(DAHDI_FLAGBIT_BRIDCHAN, &dchan->flags);
-#endif
 		clear_bit(DAHDI_FLAGBIT_HDLC, &dchan->flags);
 	}
 	return 0;
@@ -1258,7 +1114,8 @@ static int bri_startup(struct dahdi_span *span)
  */
 static int bri_shutdown(struct dahdi_span *span)
 {
-	xpd_t			*xpd = span->pvt;
+	struct phonedev	*phonedev = container_of(span, struct phonedev, span);
+	xpd_t		*xpd = container_of(phonedev, struct xpd, phonedev);
 	struct BRI_priv_data	*priv;
 
 	BUG_ON(!xpd);
@@ -1270,11 +1127,12 @@ static int bri_shutdown(struct dahdi_span *span)
 	}
 	XPD_DBG(GENERAL, xpd, "SHUTDOWN\n");
 	// Turn off all channels
-	CALL_XMETHOD(XPD_STATE, xpd->xbus, xpd, 0);
+	CALL_PHONE_METHOD(card_state, xpd, 0);
 	return 0;
 }
 
-void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
+static void BRI_card_pcm_recompute(xpd_t *xpd,
+		xpp_line_t dont_care)
 {
 	int		i;
 	int		line_count;
@@ -1284,7 +1142,7 @@ void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
 	unsigned long	flags;
 
 	BUG_ON(!xpd);
-	main_xpd = xpd_byaddr(xbus, xpd->addr.unit, 0);
+	main_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, 0);
 	if(!main_xpd) {
 		XPD_DBG(DEVICES, xpd, "Unit 0 is already gone. Ignore request\n");
 		return;
@@ -1293,15 +1151,15 @@ void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
 	 * We calculate all subunits, so use the main lock
 	 * as a mutex for the whole operation.
 	 */
-	spin_lock_irqsave(&main_xpd->lock_recompute_pcm, flags);
+	spin_lock_irqsave(&PHONEDEV(main_xpd).lock_recompute_pcm, flags);
 	line_count = 0;
 	pcm_mask = 0;
 	for(i = 0; i < MAX_SUBUNIT; i++) {
-		xpd_t		*sub_xpd = xpd_byaddr(xbus, main_xpd->addr.unit, i);
+		xpd_t		*sub_xpd = xpd_byaddr(xpd->xbus, main_xpd->addr.unit, i);
 
 		if(sub_xpd) {
 			xpp_line_t	lines =
-				sub_xpd->offhook_state & ~sub_xpd->digital_signalling;
+				PHONEDEV(sub_xpd).offhook_state & ~(PHONEDEV(sub_xpd).digital_signalling);
 
 			if(lines) {
 				pcm_mask |= PCM_SHIFT(lines, i);
@@ -1328,10 +1186,10 @@ void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
 		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) + line_count * DAHDI_CHUNKSIZE
 		: 0L;
 	update_wanted_pcm_mask(main_xpd, pcm_mask, pcm_len);
-	spin_unlock_irqrestore(&main_xpd->lock_recompute_pcm, flags);
+	spin_unlock_irqrestore(&PHONEDEV(main_xpd).lock_recompute_pcm, flags);
 }
 
-static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void BRI_card_pcm_fromspan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	unsigned long	flags;
@@ -1341,18 +1199,17 @@ static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	xpp_line_t	wanted_lines;
 
 
-	BUG_ON(!xbus);
 	BUG_ON(!xpd);
 	BUG_ON(!pack);
 	pcm = RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, pcm);
 	for(subunit = 0; subunit < MAX_SUBUNIT; subunit++) {
 		xpd_t		*tmp_xpd;
 
-		tmp_xpd = xpd_byaddr(xbus, xpd->addr.unit, subunit);
+		tmp_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, subunit);
 		if(!tmp_xpd || !tmp_xpd->card_present)
 			continue;
 		spin_lock_irqsave(&tmp_xpd->lock, flags);
-		wanted_lines = tmp_xpd->wanted_pcm_mask;
+		wanted_lines = PHONEDEV(tmp_xpd).wanted_pcm_mask;
 		for_each_line(tmp_xpd, i) {
 			struct dahdi_chan	*chan = XPD_CHAN(tmp_xpd, i);
 
@@ -1378,7 +1235,7 @@ static void BRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	RPACKET_FIELD(pack, GLOBAL, PCM_WRITE, lines) = pcm_mask;
 }
 
-static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
+static void BRI_card_pcm_tospan(xpd_t *xpd, xpacket_t *pack)
 {
 	byte		*pcm;
 	xpp_line_t	pcm_mask;
@@ -1400,7 +1257,7 @@ static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 
 		if(!pcm_mask)
 			break;	/* optimize */
-		tmp_xpd = xpd_byaddr(xbus, xpd->addr.unit, subunit);
+		tmp_xpd = xpd_byaddr(xpd->xbus, xpd->addr.unit, subunit);
 		if(!tmp_xpd || !tmp_xpd->card_present || !SPAN_REGISTERED(tmp_xpd))
 			continue;
 		spin_lock_irqsave(&tmp_xpd->lock, flags);
@@ -1420,27 +1277,48 @@ static void BRI_card_pcm_tospan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	}
 }
 
-/*---------------- BRI: HOST COMMANDS -------------------------------------*/
-
-static /* 0x0F */ HOSTCMD(BRI, XPD_STATE, bool on)
+static int BRI_timing_priority(xpd_t *xpd)
 {
 	struct BRI_priv_data	*priv;
 
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	if (priv->layer1_up)
+		return PHONEDEV(xpd).timing_priority;
+	XPD_DBG(SYNC, xpd, "No timing priority (no layer1)\n");
+	return -ENOENT;
+}
+
+int BRI_echocancel_timeslot(xpd_t *xpd, int pos)
+{
+	return xpd->addr.subunit * 4 + pos;
+}
+
+static int BRI_echocancel_setmask(xpd_t *xpd, xpp_line_t ec_mask)
+{
+	struct BRI_priv_data	*priv;
+	int i;
+
 	BUG_ON(!xpd);
 	priv = xpd->priv;
-	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
-	if(on) {
-		if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags)) {
-			if(xpd->direction == TO_PSTN)
-				te_activation(xpd, 1);
-			else
-				nt_activation(xpd, 1);
-		}
-	} else if(IS_NT(xpd))
-		nt_activation(xpd, 0);
+	BUG_ON(!priv);
+	XPD_DBG(GENERAL, xpd, "0x%8X\n", ec_mask);
+	if (!ECHOOPS(xpd->xbus)) {
+		XPD_DBG(GENERAL, xpd,
+				"No echo canceller in XBUS: Doing nothing.\n");
+		return -EINVAL;
+	}
+	for (i = 0; i < PHONEDEV(xpd).channels - 1; i++) {
+		int	on = BIT(i) & ec_mask;
+
+		CALL_EC_METHOD(ec_set, xpd->xbus, xpd, i, on);
+	}
+	CALL_EC_METHOD(ec_update, xpd->xbus, xpd->xbus);
 	return 0;
 }
 
+/*---------------- BRI: HOST COMMANDS -------------------------------------*/
+
 static /* 0x33 */ HOSTCMD(BRI, SET_LED, enum bri_led_names which_led, enum led_state to_led_state)
 {
 	int			ret = 0;
@@ -1487,14 +1365,12 @@ static int write_state_register(xpd_t *xpd, byte value)
 /*---------------- BRI: Astribank Reply Handlers --------------------------*/
 static void su_new_state(xpd_t *xpd, byte reg_x30)
 {
-	xbus_t			*xbus;
 	struct BRI_priv_data	*priv;
 	su_rd_sta_t		new_state;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
 	BUG_ON(!priv);
-	xbus = xpd->xbus;
 	if(!priv->initialized) {
 		XPD_ERR(xpd, "%s called on uninitialized AB\n", __FUNCTION__);
 		return;
@@ -1624,7 +1500,7 @@ static int BRI_card_register_reply(xbus_t *xbus, xpd_t *xpd, reg_cmd_t *info)
 		ret = rx_dchan(xpd, info);
 		if (ret < 0) {
 			priv->dchan_rx_drops++;
-			if(atomic_read(&xpd->open_counter) > 0)
+			if(atomic_read(&PHONEDEV(xpd).open_counter) > 0)
 				XPD_NOTICE(xpd, "Multibyte Drop: errno=%d\n", ret);
 		} 
 		goto end;
@@ -1646,6 +1522,49 @@ end:
 	return 0;
 }
 
+static int BRI_card_state(xpd_t *xpd, bool on)
+{
+	struct BRI_priv_data	*priv;
+
+	BUG_ON(!xpd);
+	priv = xpd->priv;
+	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
+	if(on) {
+		if(!test_bit(HFC_L1_ACTIVATED, &priv->l1_flags)) {
+			if( ! IS_NT(xpd))
+				te_activation(xpd, 1);
+			else
+				nt_activation(xpd, 1);
+		}
+	} else if(IS_NT(xpd))
+		nt_activation(xpd, 0);
+	return 0;
+}
+
+static const struct xops	bri_xops = {
+	.card_new	= BRI_card_new,
+	.card_init	= BRI_card_init,
+	.card_remove	= BRI_card_remove,
+	.card_tick	= BRI_card_tick,
+	.card_register_reply	= BRI_card_register_reply,
+};
+
+static const struct phoneops	bri_phoneops = {
+	.card_dahdi_preregistration	= BRI_card_dahdi_preregistration,
+	.card_dahdi_postregistration	= BRI_card_dahdi_postregistration,
+	.card_hooksig	= BRI_card_hooksig,
+	.card_pcm_recompute	= BRI_card_pcm_recompute,
+	.card_pcm_fromspan	= BRI_card_pcm_fromspan,
+	.card_pcm_tospan	= BRI_card_pcm_tospan,
+	.card_timing_priority	= BRI_timing_priority,
+	.echocancel_timeslot	= BRI_echocancel_timeslot,
+	.echocancel_setmask	= BRI_echocancel_setmask,
+	.card_ioctl	= BRI_card_ioctl,
+	.card_open	= BRI_card_open,
+	.card_close	= BRI_card_close,
+	.card_state	= BRI_card_state,
+};
+
 static xproto_table_t PROTO_TABLE(BRI) = {
 	.owner = THIS_MODULE,
 	.entries = {
@@ -1654,24 +1573,8 @@ static xproto_table_t PROTO_TABLE(BRI) = {
 	.name = "BRI",	/* protocol name */
 	.ports_per_subunit = 1,
 	.type = XPD_TYPE_BRI,
-	.xops = {
-		.card_new	= BRI_card_new,
-		.card_init	= BRI_card_init,
-		.card_remove	= BRI_card_remove,
-		.card_dahdi_preregistration	= BRI_card_dahdi_preregistration,
-		.card_dahdi_postregistration	= BRI_card_dahdi_postregistration,
-		.card_hooksig	= BRI_card_hooksig,
-		.card_tick	= BRI_card_tick,
-		.card_pcm_recompute	= BRI_card_pcm_recompute,
-		.card_pcm_fromspan	= BRI_card_pcm_fromspan,
-		.card_pcm_tospan	= BRI_card_pcm_tospan,
-		.card_ioctl	= BRI_card_ioctl,
-		.card_open	= BRI_card_open,
-		.card_close	= BRI_card_close,
-		.card_register_reply	= BRI_card_register_reply,
-
-		.XPD_STATE	= XPROTO_CALLER(BRI, XPD_STATE),
-	},
+	.xops = &bri_xops,
+	.phoneops = &bri_phoneops,
 	.packet_is_valid = bri_packet_is_valid,
 	.packet_dump = bri_packet_dump,
 };
@@ -1690,6 +1593,7 @@ static void bri_packet_dump(const char *msg, xpacket_t *pack)
 }
 /*------------------------- REGISTER Handling --------------------------*/
 
+#ifdef	CONFIG_PROC_FS
 static int proc_bri_info_read(char *page, char **start, off_t off, int count, int *eof, void *data)
 {
 	int			len = 0;
@@ -1715,7 +1619,7 @@ static int proc_bri_info_read(char *page, char **start, off_t off, int count, in
 					priv->state_register.bits.v_su_info0,
 					priv->state_register.bits.v_g2_g3);
 	} else
-		len += sprintf(page + len, "Unkown\n");
+		len += sprintf(page + len, "Unknown\n");
 	if(IS_NT(xpd)) {
 		len += sprintf(page + len, "T1 Timer: %d\n", priv->t1);
 	} else {
@@ -1732,9 +1636,6 @@ static int proc_bri_info_read(char *page, char **start, off_t off, int count, in
 	} else {
 		len += sprintf(page + len, "(dead)\n");
 	}
-#ifndef	CONFIG_DAHDI_BRI_DCHANS
-	len += sprintf(page + len, "hdlc_pending=%d\n", atomic_read(&priv->hdlc_pending));
-#endif
 	len += sprintf(page + len, "dchan_notx_ticks: %d\n", priv->dchan_notx_ticks);
 	len += sprintf(page + len, "dchan_norx_ticks: %d\n", priv->dchan_norx_ticks);
 	len += sprintf(page + len, "LED: %-10s = %d\n", "GREEN", priv->ledstate[GREEN_LED]);
@@ -1752,22 +1653,7 @@ static int proc_bri_info_read(char *page, char **start, off_t off, int count, in
 		len = 0;
 	return len;
 }
-
-static DRIVER_ATTR_READER(dchan_hardhdlc_show, drv,buf)
-{
-	int			len = 0;
-
-#if	defined(CONFIG_DAHDI_BRI_DCHANS)
-	len += sprintf(buf + len, "0\n");
-#elif	defined(DAHDI_SIG_HARDHDLC)
-	len += sprintf(buf + len, "1\n");
-#else
-#error Cannot build BRI without BRISTUFF or HARDHDLC supprt
 #endif
-	return len;
-}
-
-static	DRIVER_ATTR(dchan_hardhdlc,S_IRUGO,dchan_hardhdlc_show,NULL);
 
 static int bri_xpd_probe(struct device *dev)
 {
@@ -1811,18 +1697,7 @@ static int __init card_bri_startup(void)
 
 	if((ret = xpd_driver_register(&bri_driver.driver)) < 0)
 		return ret;
-	ret = driver_create_file(&bri_driver.driver, &driver_attr_dchan_hardhdlc);
-	if(ret < 0)
-		return ret;
 	INFO("revision %s\n", XPP_VERSION);
-#if	defined(CONFIG_DAHDI_BRI_DCHANS)
-	INFO("FEATURE: WITH BRISTUFF\n");
-#elif	defined(DAHDI_SIG_HARDHDLC)
-	INFO("FEATURE: WITH HARDHDLC\n");
-#else
-#error Cannot build BRI without BRISTUFF or HARDHDLC supprt
-#endif
-
 	xproto_register(&PROTO_TABLE(BRI));
 	return 0;
 }
@@ -1831,7 +1706,6 @@ static void __exit card_bri_cleanup(void)
 {
 	DBG(GENERAL, "\n");
 	xproto_unregister(&PROTO_TABLE(BRI));
-	driver_remove_file(&bri_driver.driver, &driver_attr_dchan_hardhdlc);
 	xpd_driver_unregister(&bri_driver.driver);
 }
 
